import { CodeBuild } from 'aws-sdk'

export const pipelineDeploymentDisabled = [
  'PROVISIONING',
  'PROVISION_REQUEST',
  'FAILED_TO_PROVISION',
  'PRE_PROVISIONED',
  'DELETING',
  'DELETED',
  'SCHEDULED_FOR_DELETION',
  'DELETION_REQUEST',
  'CREATED',
]
export const pipelineNotDeletable = [
  'IN_PROGRESS',
  'DELETING',
  'PROVISION_REQUEST',
  'PROVISIONING',
  'QUEUED',
  'SCHEDULED_FOR_DELETION',
  'DELETION_REQUEST',
  'CREATED',
  'DELETED',
]

export const pipelineProvisioning = ['PROVISIONING', 'PROVISION_REQUEST']

export const pipelinePreprovisionedFlow = ['PRE_PROVISIONED', 'PROVISIONING', 'PROVISION_REQUEST', 'FAILED_TO_PROVISION']

export type PipelineProvisionBuildStatuses =
  | 'PRE_PROVISIONED'
  | 'PROVISIONING'
  | 'PROVISION_REQUEST'
  | 'FAILED_TO_PROVISION'

export type PipelineDeployingBuildStatues = 'QUEUED' | 'COMPLETED' | 'FAILED'

export type PipelineDeleteBuildStatuses = 'DELETED' | 'DELETING' | 'SCHEDULED_FOR_DELETION' | 'DELETION_REQUEST'

export type PipelineBuildStatus =
  | 'CREATED'
  | ('READY_FOR_DEPLOYMENT' &
      PipelineDeleteBuildStatuses &
      PipelineDeployingBuildStatues &
      PipelineProvisionBuildStatuses)
  | CodeBuild.StatusType


export interface GithubRepositoryInterface {
  repositoryUrl?: string
  installationId?: number
  repositoryId?: number
}

export interface PipelineModelInterface {
  id?: string
  name?: string
  organisationId?: string
  created?: string
  modified?: string
  developerId?: string
  appType?: AppTypeEnum
  buildCommand?: string
  testCommand?: string
  packageManager?: PackageManagerEnum
  repository?: GithubRepositoryInterface
  branch?: string
  outDir?: string
  buildStatus?: PipelineBuildStatus | string
  appId?: string
  subDomain?: string
  bitbucketClientId?: string
}

export interface PipelineRunnerModelInterface {
  id?: string
  pipelineId?: string
  created?: string
  modified?: string
  appType?: AppTypeEnum
  type?: PipelineRunnerType
  buildCommand?: string
  packageManager?: PackageManagerEnum
  tasks?: TaskModelInterface[]
  buildStatus?: CodeBuild.StatusType
  s3BuildLogsLocation?: string
  buildVersion?: string
  currentlyDeployed?: boolean
}

export interface TaskModelInterface {
  id?: string
  pipelineRunnerId?: string
  created?: string
  modified?: string
  functionName?: CodeBuild.BuildPhaseType
  buildStatus?: CodeBuild.StatusType
  startTime?: Date
  endTime?: Date
  elapsedTime?: string
}

export enum PackageManagerEnum {
  NPM = 'npm',
  YARN = 'yarn',
  YARN_BERRY = 'yarn-berry',
}

export enum AppTypeEnum {
  NODE = 'node',
  REACT = 'react',
}

export enum PipelineRunnerType {
  BUILD = 'BUILD',
  RELEASE = 'RELEASE',
  REPO = 'REPO',
}
